// vim: set syntax=cpp :

template<unsigned int N> class TaskTpl : public Task {

	char m_input_data_path[1024];

	// used to pass data into CEC05 functions
	long double * m_x;
	unsigned int m_size;

	// gathering statistics
	long double m_bestEvaluation;
	unsigned int m_invocationCounter;

	long double m_optimum;		// known optimal value
	long double m_acc;			// accuracy level
	unsigned int m_accFES;		// when this accuracy level was reached
	unsigned int m_convStep;	// record best solution every `X' evaluations

	const long double ACC;

	// Record function error value (f(x)-f(x*)) after 1e3, 1e4, 1e5 FES and at termination (due
	// to Ter_Err or Max_FES) for each run.
	std::ofstream m_out1;
	// Record the FES needed in each run to achieve the following fixed accuracy level. The
	// Max_FES applies.
	std::ofstream m_out2;
	// Convergence Graphs (or Run-length distribution graphs)
	std::ofstream m_out4;

	bool m_1e3, m_1e4, m_1e5, m_accLevel;
	std::list<unsigned int> m_evaluationAt;
	std::list<long double> m_evaluationVal;

	// -----------------------------------------------------------------

	void openFile (std::ofstream & f, unsigned int _c) {
		std::stringstream name;
		name << "cec05_fun" << N << "_crit" << _c << ".csv";
		f.open(name.str().c_str(), std::ios_base::app);
	}

	// -----------------------------------------------------------------

	void initializeStatistics () {
		m_optimum = optima_values[N-1];
		m_acc = m_optimum + accuracy[N-1];

		m_bestEvaluation = std::numeric_limits<long double>::max();

		openFile(m_out1, 1);
		openFile(m_out2, 2);
		openFile(m_out4, 4);

		m_out1.setf(std::ios::fixed,std::ios::floatfield);
		m_out1.precision(15);

		m_out4.precision(10);

		m_1e3 = m_1e4 = m_1e5 = m_accLevel = false;
		m_accFES = 0;
		m_convStep = m_size * 100; // this should give 100 record points
	}

	// -----------------------------------------------------------------

	void gatherStatistics(const double * _point, long double _evaluation) {
		if (_evaluation < m_bestEvaluation)
			m_bestEvaluation = _evaluation;

		// criterion one
#define CRIT1(cnt, flag)                                \
		if (this->m_invocationCounter > cnt and not flag) { \
			m_out1 << m_bestEvaluation << ' ';          \
			flag = true;                               \
		}
		CRIT1(1000, m_1e3)
		CRIT1(10000, m_1e4)
		CRIT1(100000, m_1e5)

//		std::cout.setf(std::ios::fixed,std::ios::floatfield);
//		std::cout.precision(15);
//		std::cout << m_bestEvaluation * ACC << ' ' << m_acc << '\n';

		// criterion two
		if (m_bestEvaluation <= m_acc and not m_accLevel) {
			m_accFES = m_invocationCounter;
			m_accLevel = true;
		}

		// criterion four
		if ((m_invocationCounter % m_convStep) == 0) {
			m_evaluationAt.push_back(m_invocationCounter);
			m_evaluationVal.push_back(m_bestEvaluation-m_optimum);
		}
	}

	// -----------------------------------------------------------------

	void finalizeStatistics () {
		// add also the final evaluation
		m_out1 << m_bestEvaluation << "\n";
		m_out2 << (m_accFES ? (int)m_accFES : -1) << "\n";

		// write convergence info
//		for (std::list<unsigned int>::iterator ai = m_evaluationAt.begin(); ai!=m_evaluationAt.end(); ++ai)
//			m_out4 << *ai << ' ';
//		m_out4 << m_invocationCounter << '\n';
		for (std::list<long double>::iterator vi = m_evaluationVal.begin(); vi!=m_evaluationVal.end(); ++vi)
			m_out4 << *vi << ' ';
		m_out4 << m_bestEvaluation-m_optimum << '\n';

		// close output streams
		m_out1.close();
		m_out2.close();
		m_out4.close();
	}

public:

	TaskTpl (unsigned int _dim, unsigned int _seed, const char * _dpath,
				int & nfunc, int & nreal, long double & seed)
		: ACC(10e6), m_invocationCounter(0)
	{
		memset(m_input_data_path, 0, 1024);
		if (_dpath)
			strncpy(m_input_data_path, _dpath, 1024);
		input_data_dir = m_input_data_path;

		nfunc = (N < 15) ? 1 : 10;
		nreal = _dim;
		seed  = _seed / (double)std::numeric_limits<unsigned int>::max();
		randomize();
		initrandomnormaldeviate();
		allocate_memory();
		initialize();
		if (N > 14)
			calc_benchmark_norm();
		m_x = new long double [2*_dim];
		m_size = _dim;

		initializeStatistics();
	}

	// -----------------------------------------------------------------

	virtual ~TaskTpl () {
		free_memory();
		delete [] m_x;

		finalizeStatistics();
	}

	// -----------------------------------------------------------------

	virtual double evaluate (const double * _point, unsigned int _size) {
		if (m_size != _size)
			report_error_ptr("wrong number of dimensions");

		long double * xx = m_x;
		const double * pp = _point, * ppend = _point+_size;
		for (; pp<ppend; ++pp, ++xx)
			*xx = *pp;
		long double evaluation = calc_benchmark_func(m_x);

		gatherStatistics(_point, evaluation);

		return evaluation;
	}

	virtual unsigned int getTaskNo () const {
		return N;
	}

};

