

#ifndef WRAPPER_H__
#define WRAPPER_H__

#ifdef __cplusplus
extern "C" {
#endif

/// Error reporting function - typedef.
typedef void (*error_func_ptr) (const char * fmt, ...);
/// Error reporting function - global pointer.
extern error_func_ptr report_error_ptr;
/**
* @brief Default error reporting function: prints the message
* and calls exit().
* @param fmt Format string.
* @param ... Additional parameters.
*/
void report_error (const char * fmt, ...);

/// Array with minial values for each function.
extern long double optima_values[25];
/// Required accuracy for each function: (optima_values[i]-best_x) < accuracy[i]
extern long double accuracy[25];

/**
 * @brief Initializes data structures. Has to be mached with call to
 * destroy_benchmark().
 *
 * @param _no   Function number.
 * @param _dim  Number of dimensions.
 * @param _seed RNG seed.
 */
int initialize_benchmark (int _no, unsigned int _dim, unsigned int _seed);
/**
 * @brief Calculates the function value for given _point. Has to be
 * preceded by a call to initialize_benchmark().
 *
 * @param _point Point data.
 * @param _size  Point size; has to be equal to _dim.
 * @return Objective function value.
 */
double evaluate_benchmark (const double * _point, unsigned int _size);
/**
 * @brief Destroys internal data structures. Has to be called
 * for each call to initialize_benchmark().
 */
int destroy_benchmark ();
/**
 * @brief Returns the optimum value for the currently chosen objective
 * function. Has to be preceded by a call to initialize_benchmark().
 */
long double get_optimum_value ();
/**
 * @brief Returns the required accuracy level for the currently chosen
 * objective function. Has to be preceded by a call to initialize_benchmark().
 */
long double get_accuracy ();

/// Returns the size of the initialization range vectors.
unsigned int get_init_range_size ();
/**
 * @brief Returns the initialization bounds vectors.
 *
 * @param _lower Buffer for the lower bounds vector.
 * @param _upper Buffer for the upper bounds vector.
 * @param _size  Size of each buffer (number of elements); has to match
 *               the value returned by get_init_range_size().
 */
void get_init_range (double * _lower, double * _upper, unsigned int _size);
/// Returns the size of the search range vectors.
unsigned int get_search_range_size ();
/**
 * @brief Returns the search bounds vectors.
 *
 * @param _lower Buffer for the lower bounds vector.
 * @param _upper Buffer for the upper bounds vector.
 * @param _size  Size of each buffer (number of elements); has to match
 *               the value returned by get_search_range_size().
 */
void get_search_range (double * _lower, double * _upper, unsigned int _size);

/**
 * @brief Where to look the the 'input_data' files.
 *
 * @param _path Path to data files.
 */
void set_input_data_dir (const char * _path);

#ifdef __cplusplus
} // extern "C"
#endif

#endif // WRAPPER_H__

